# -*- coding: utf-8 -*-

# Form implementation generated from reading ui file '/home/flavio/Documents/Projetos/Epigrass/Epigrass-devel-qt4/display.ui'
#
# Created: Mon Jun 23 12:11:02 2008
#      by: PyQt4 UI code generator 4.3.3
#
# WARNING! All changes made in this file will be lost!

from PyQt4 import QtCore, QtGui

class Ui_Form(object):
    def setupUi(self, Form):
        Form.setObjectName("Form")
        Form.resize(QtCore.QSize(QtCore.QRect(0,0,797,739).size()).expandedTo(Form.minimumSizeHint()))

        sizePolicy = QtGui.QSizePolicy(QtGui.QSizePolicy.Expanding,QtGui.QSizePolicy.Expanding)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(Form.sizePolicy().hasHeightForWidth())
        Form.setSizePolicy(sizePolicy)

        self.gridlayout = QtGui.QGridLayout(Form)
        self.gridlayout.setObjectName("gridlayout")

        self.vboxlayout = QtGui.QVBoxLayout()
        self.vboxlayout.setObjectName("vboxlayout")

        self.label = QtGui.QLabel(Form)
        self.label.setObjectName("label")
        self.vboxlayout.addWidget(self.label)

        self.hboxlayout = QtGui.QHBoxLayout()
        self.hboxlayout.setObjectName("hboxlayout")

        self.splitter = QtGui.QSplitter(Form)
        self.splitter.setOrientation(QtCore.Qt.Vertical)
        self.splitter.setObjectName("splitter")

        self.mapView = QtGui.QGraphicsView(self.splitter)
        self.mapView.setProperty("cursor",QtCore.QVariant(QtCore.Qt.PointingHandCursor))
        self.mapView.setAutoFillBackground(True)
        self.mapView.setObjectName("mapView")

        self.layoutWidget = QtGui.QWidget(self.splitter)
        self.layoutWidget.setObjectName("layoutWidget")

        self.vboxlayout1 = QtGui.QVBoxLayout(self.layoutWidget)
        self.vboxlayout1.setObjectName("vboxlayout1")

        self.horizontalSlider = QtGui.QSlider(self.layoutWidget)
        self.horizontalSlider.setOrientation(QtCore.Qt.Horizontal)
        self.horizontalSlider.setTickPosition(QtGui.QSlider.TicksBelow)
        self.horizontalSlider.setTickInterval(0)
        self.horizontalSlider.setObjectName("horizontalSlider")
        self.vboxlayout1.addWidget(self.horizontalSlider)

        self.hboxlayout1 = QtGui.QHBoxLayout()
        self.hboxlayout1.setObjectName("hboxlayout1")

        self.label_2 = QtGui.QLabel(self.layoutWidget)

        sizePolicy = QtGui.QSizePolicy(QtGui.QSizePolicy.Fixed,QtGui.QSizePolicy.Preferred)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.label_2.sizePolicy().hasHeightForWidth())
        self.label_2.setSizePolicy(sizePolicy)
        self.label_2.setObjectName("label_2")
        self.hboxlayout1.addWidget(self.label_2)

        spacerItem = QtGui.QSpacerItem(461,23,QtGui.QSizePolicy.Expanding,QtGui.QSizePolicy.Minimum)
        self.hboxlayout1.addItem(spacerItem)

        self.lcdNumber = QtGui.QLCDNumber(self.layoutWidget)
        self.lcdNumber.setObjectName("lcdNumber")
        self.hboxlayout1.addWidget(self.lcdNumber)
        self.vboxlayout1.addLayout(self.hboxlayout1)

        self.qwtPlot = QwtPlot(self.layoutWidget)
        self.qwtPlot.setObjectName("qwtPlot")
        self.vboxlayout1.addWidget(self.qwtPlot)
        self.hboxlayout.addWidget(self.splitter)

        self.vboxlayout2 = QtGui.QVBoxLayout()
        self.vboxlayout2.setSizeConstraint(QtGui.QLayout.SetDefaultConstraint)
        self.vboxlayout2.setObjectName("vboxlayout2")

        self.pushButton = QtGui.QPushButton(Form)

        sizePolicy = QtGui.QSizePolicy(QtGui.QSizePolicy.Fixed,QtGui.QSizePolicy.Fixed)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.pushButton.sizePolicy().hasHeightForWidth())
        self.pushButton.setSizePolicy(sizePolicy)
        self.pushButton.setObjectName("pushButton")
        self.vboxlayout2.addWidget(self.pushButton)

        self.pushButton_2 = QtGui.QPushButton(Form)

        sizePolicy = QtGui.QSizePolicy(QtGui.QSizePolicy.Fixed,QtGui.QSizePolicy.Fixed)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.pushButton_2.sizePolicy().hasHeightForWidth())
        self.pushButton_2.setSizePolicy(sizePolicy)
        self.pushButton_2.setObjectName("pushButton_2")
        self.vboxlayout2.addWidget(self.pushButton_2)

        spacerItem1 = QtGui.QSpacerItem(20,521,QtGui.QSizePolicy.Minimum,QtGui.QSizePolicy.Expanding)
        self.vboxlayout2.addItem(spacerItem1)
        self.hboxlayout.addLayout(self.vboxlayout2)
        self.vboxlayout.addLayout(self.hboxlayout)
        self.gridlayout.addLayout(self.vboxlayout,0,0,1,1)

        self.retranslateUi(Form)
        QtCore.QMetaObject.connectSlotsByName(Form)

    def retranslateUi(self, Form):
        Form.setWindowTitle(QtGui.QApplication.translate("Form", "Simulation Display", None, QtGui.QApplication.UnicodeUTF8))
        self.label.setText(QtGui.QApplication.translate("Form", "Geographical View", None, QtGui.QApplication.UnicodeUTF8))
        self.label_2.setText(QtGui.QApplication.translate("Form", "Time Series", None, QtGui.QApplication.UnicodeUTF8))
        self.pushButton.setText(QtGui.QApplication.translate("Form", "Replay", None, QtGui.QApplication.UnicodeUTF8))
        self.pushButton_2.setText(QtGui.QApplication.translate("Form", "Stop", None, QtGui.QApplication.UnicodeUTF8))

from PyQt4.Qwt5 import QwtPlot


if __name__ == "__main__":
    import sys
    app = QtGui.QApplication(sys.argv)
    Form = QtGui.QWidget()
    ui = Ui_Form()
    ui.setupUi(Form)
    Form.show()
    sys.exit(app.exec_())



#!/usr/bin/env python

#program to play simulations from database
import dgraph, cPickle, glob, os, ogr,  time
from math import *
try:
    from PyQt4.QtGui import *
except ImportError: 
    print "Please install PyQT 4"
#
from numpy import *
import sqlobject as SO

from matplotlib import cm


class viewer:
    """
    """
    def __init__(self, host='localhost', port=3306, user='epigrass', pw='epigrass', db='epigrass',backend='mysql',encoding='latin-1', gui=None):
        self.host = host
        self.port = port
        self.user = user
        self.pw = pw
        self.db = db
        self.backend = backend
        self.encoding = encoding
        self.gui = gui
        
        if backend == 'sqlite':
            db_filename = os.path.abspath('Epigrass.sqlite')
            connection_string = 'sqlite:' + db_filename
        elif backend == 'mysql':
            connection_string = r'%s://%s:%s@%s/%s'%(backend,user,pw,host,db)
        elif backend == 'csv':
            pass
        else:
            sys.exit('Invalid Database Backend specified: %s'%backend)
        if not backend == 'csv':
            self.connection = SO.connectionForURI(connection_string)
        
        self.dmap = 0#int(input('Draw Map?(0,1) '))
        self.tables = self.getTables()

        #self.nodes = self.readNodes(self.tables[table])
        #self.numbnodes = len(self.nodes)
        #self.data = self.readData(self.tables[table])
        #self.numbsteps = len(self.data)/self.numbnodes
        #self.viewGraph()
        
    def getTables(self):
        """
        Returns list of table names from current database connection
        """
        if self.backend == 'sqlite':
            r = [i[0] for i in self.connection.queryAll("select name from sqlite_master where type='table';")]
        elif self.backend == 'mysql':
            r = [i[0] for i in self.connection.queryAll('SHOW TABLES')]
        elif self.backend =='csv':
            r = glob.glob('*.tab')
        
        return r
    def getFields(self,table):
        """
        Returns a list of fields (column names) for a given table.
        table is a string with table name
        """
        if self.backend == 'sqlite':
            r = [i[1] for i in self.connection.queryAll('PRAGMA table_info(%s)'%table)]
            shp =self.connection.queryAll('SELECT the_world$shapefile FROM %s'%(table+'_meta'))
            print shp
            self.shapefile = eval(shp[0][0])
            print self.shapefile
        elif self.backend == 'mysql':
            r = [i[0] for i in self.connection.queryAll('SHOW FIELDS FROM %s'%table)]
            shp =self.connection.queryAll('SELECT the_world$shapefile FROM %s'%(table+'_meta'))
            self.shapefile = eval(shp[0][0])
        elif self.backend == 'csv':
            f = open(table,'r')
            r = f.read().strip().split(',')
            f.close()

        #select only the key names (each element in r is a tuple, containing
        # name and field descriptors )
        
        return r
    
    def readNodes(self,name,table):
        """
        Reads geocode and coords from database table
        for each node and adjacency matrix.
        """
        if self.backend =="csv":
            # the equivalent of a select for a csv file
            f=open(table,"r")
            names = f.readline().strip().split(",")#remove header
            d = {}
            for l in f:
                l=l.strip().split(',')
                d[l[names.index('geocode')]]=[l[names.index('geocode')],l[names.index('lat')],l[names.index('longit')],l[names.index('name')]]
            r= d.values()
            f.close()
            self.numbnodes = len(r)
        else:
            r = self.connection.queryAll('SELECT geocode,lat,longit,name FROM %s WHERE time = 0'%table)
            self.numbnodes = len(r)
        self.nodes_pos = r#[(i[1], i[2], i[0], i[3])for i in r]
        self.nodes_gc = [i[0] for i in r]
            
        # get adjacency matrix
#        if not os.getcwd() == self.
        file = open('adj_'+name)
        m = cPickle.load(file)
        self.adjacency = m
        file.close()
        return r,m
    
    
    def readData(self,table):
        """
        read node time series data
        """
        if self.backend =="csv":
            f=open(table,"r")
            f.readline()#remove header
            r = []
            for l in f:
                l=l.strip().split(',')
                r.append(l)
            f.close()
        else:
            r = self.connection.queryAll('SELECT * FROM %s'%table)
        return r

    def readEdges(self,table):
        """
        Read edge time series
        """
        if self.backend =="csv":
            tab = table.split(".")[0]+"_e.tab"
            f = open(tab,"r")
            f.readline()#remove header
            r = [l.strip().split(',') for l in f]
            f.close()
            self.numbedges = len(r)
        else:
            tab = table+'e'
            r = self.connection.queryAll('SELECT * FROM %s'%tab)
            self.numbedges = len(r)
        self.elist = [(self.nodes_gc.index(e[0]), self.nodes_gc.index(e[1])) for e in r]
        if not self.elist:
            self.elist = transpose(self.adjacency.nonzero()).tolist()
        return r
        
    def viewGraph(self, nodes, am, var,mapa=''):
        """
        Starts the Qt display of the map or graph.
        """
        if self.shapefile:
            self.gui.openGraphDisplay(mapa,self.shapefile[1],self.shapefile[2], self.nodes_pos, self.elist  )
        else:
            self.gui.openGraphDisplay(mapa,nlist=self.nodes_pos, elist=self.elist  )
        self.gr = self.gui.graphDisplay
        self.gr.qwtPlot.setTitle(var)

            
    def anim(self,data,edata, numbsteps,pos, rate=20):
        """
        Starts the animation.
        * data: time series from database
        * edata: infectious traveling for edge painting
        * pos: column number of variable to animate
        """
        #FIXME: the animation rate is not working....
        self.gr.horizontalSlider.setEnabled(0)
        self.gr.horizontalSlider.setMaximum(numbsteps)
        for t in xrange(numbsteps):
            stepdict = {}
            for i in xrange(self.numbnodes):
                start = i*numbsteps+t
                stepdict[data[start][0]] = data[start][pos] #{geocode: value}
            self.gr.drawStep(t, stepdict)
            #paint Edges when there are infectious coming or going 
            if not edata:
                continue
            elist = []
            for i in xrange(self.numbedges):
                start = i*numbsteps+t
                if edata[start][-1]+edata[start][-2]:
                    elist.append(edata[start][1])
            self.gr.flashBorders(elist)
#            time.sleep(1./rate)
        self.gr.horizontalSlider.setEnabled(1)
    

    def plotTs(self,ts,name):
        """
        Uses gcurve to plot the time-series of a given city object
        """
##        try:
##            self.gg.display.visible=0
##        except:pass
        self.gg = VG.gdisplay(title='%s'%name,xtitle='time',ytitle='count')
        g=VG.gcurve(color=VG.color.green)
        for t,n in enumerate(ts):
            g.plot(pos=(t,n))
    
    def keyin(self,data,edata,numbsteps,pos,rate):
        """
        Implements keyboard and mouse interactions
        """
        while 1:
            ob = self.gr.display.mouse.pick
            try:
                ob.sn(2)
                if self.gr.display.mouse.alt and not ob.paren.tsdone:
                    self.plotTs(ob.paren.ts,ob.paren.name)
                    ob.paren.tsdone = 1
            except:pass
            if self.gr.display.mouse.clicked:
                m = self.gr.display.mouse.getclick()
                #print m.click
                loc = m.pos
                self.gr.display.center = loc
            if self.gr.display.kb.keys: # is there an event waiting to be processed?
                s = self.gr.display.kb.getkey() # obtain keyboard information
                if s == 'r': #Replay animation
                    for i in self.gr.nodes:
                        self.paintNode(0,i,numbsteps,col='g')
                        i.painted=0
                    self.anim(data,edata,numbsteps,pos,rate)
                else:
                    pass
            
    
if __name__ == "__main__":
    Display=viewer(user='root',pw='mysql')
    Display.anim()
    Display.keyin()




import sys
from PyQt4 import QtGui, QtCore, QtOpenGL
import elasticnodes as EN
import math

class MainWindow(QtGui.QGraphicsView):
    def __init__(self,parent = None,name = None,fl = 0):
        QtGui.QGraphicsView.__init__(self)

        self.timerId = 0
        scene = QtGui.QGraphicsScene(self)
        scene.setItemIndexMethod(QtGui.QGraphicsScene.NoIndex)
        scene.setSceneRect(-200, -200, 400, 400)
        self.setScene(scene)
        self.setCacheMode(QtGui.QGraphicsView.CacheBackground)
        self.setRenderHint(QtGui.QPainter.Antialiasing)
        self.setTransformationAnchor(QtGui.QGraphicsView.AnchorUnderMouse)
        self.setResizeAnchor(QtGui.QGraphicsView.AnchorViewCenter)

        self.scale(0.8, 0.8)
        self.setMinimumSize(400, 400)
        self.setWindowTitle(self.tr("Simulation Display"))


        # test code
##        self.node1 = Node(self.canvas, pos=(10,10,2))
##        self.node2 = Node(self.canvas,pos=(100,100,2))
##        self.edge = Edge(self.canvas,(10,10,100,100))
#        self.map = Map('limites.map',self.canvas)
#        self.canvas.update()
#        self.view.resize(self.view.sizeHint())
    def itemMoved(self):
        if not self.timerId:
            self.timerId = self.startTimer(1000 / 25)

    def keyPressEvent(self, event):
        key = event.key()

        if key == QtCore.Qt.Key_Up:
            self.centerNode.moveBy(0, -20)
        elif key == QtCore.Qt.Key_Down:
            self.centerNode.moveBy(0, 20)
        elif key == QtCore.Qt.Key_Left:
            self.centerNode.moveBy(-20, 0)
        elif key == QtCore.Qt.Key_Right:
            self.centerNode.moveBy(20, 0)
        elif key == QtCore.Qt.Key_Plus:
            self.scaleView(1.2)
        elif key == QtCore.Qt.Key_Minus:
            self.scaleView(1 / 1.2)
        elif key == QtCore.Qt.Key_Space or key == QtCore.Qt.Key_Enter:
            for item in self.scene().items():
                if isinstance(item, Node):
                    item.setPos(-150 + QtCore.qrand() % 300, -150 + QtCore.qrand() % 300)
        else:
            QtGui.QGraphicsView.keyPressEvent(self, event)

    def timerEvent(self, event):
        nodes = [item for item in self.scene().items() if isinstance(item, Node)]

        for node in nodes:
            node.calculateForces()

        itemsMoved = False
        for node in nodes:
            if node.advance():
                itemsMoved = True

        if not itemsMoved:
            self.killTimer(self.timerId)
            self.timerId = 0

    def wheelEvent(self, event):
        self.scaleView(math.pow(2.0, -event.delta() / 240.0))

    def drawBackground(self, painter, rect):
        # Shadow.
        sceneRect = self.sceneRect()
        rightShadow = QtCore.QRectF(sceneRect.right(), sceneRect.top() + 5, 5, sceneRect.height())
        bottomShadow = QtCore.QRectF(sceneRect.left() + 5, sceneRect.bottom(), sceneRect.width(), 5)
        if rightShadow.intersects(rect) or rightShadow.contains(rect):
            painter.fillRect(rightShadow, QtCore.Qt.darkGray)
        if bottomShadow.intersects(rect) or bottomShadow.contains(rect):
            painter.fillRect(bottomShadow, QtCore.Qt.darkGray)

        # Fill.
        gradient = QtGui.QLinearGradient(sceneRect.topLeft(), sceneRect.bottomRight())
        gradient.setColorAt(0, QtCore.Qt.white)
        gradient.setColorAt(1, QtCore.Qt.lightGray)
        painter.fillRect(rect.intersect(sceneRect), QtGui.QBrush(gradient))
        painter.setBrush(QtCore.Qt.NoBrush)
        painter.drawRect(sceneRect)

        # Text.
        textRect = QtCore.QRectF(sceneRect.left() + 4, sceneRect.top() + 4,
                                 sceneRect.width() - 4, sceneRect.height() - 4)
        message = self.tr("Click and drag the nodes around, and zoom with the "
                          "mouse wheel or the '+' and '-' keys")

        font = painter.font()
        font.setBold(True)
        font.setPointSize(14)
        painter.setFont(font)
        painter.setPen(QtCore.Qt.lightGray)
        painter.drawText(textRect.translated(2, 2), message)
        painter.setPen(QtCore.Qt.black)
        painter.drawText(textRect, message)

    def scaleView(self, scaleFactor):
        factor = self.matrix().scale(scaleFactor, scaleFactor).mapRect(QtCore.QRectF(0, 0, 1, 1)).width()

        if factor < 0.07 or factor > 100:
            return

        self.scale(scaleFactor, scaleFactor)
#class Canvas(QtGui.QCanvas):
#    pass

class Node(EN.Node):
    def __init__(self,display,width=50.,height=50., name='Node',pos=(1,1,2),color=(0.,255.,0.)):
        self.parent.__init__(display)

    def setLabel(self,canvas,text,pos):
        """
        Set the node label
        """
        x,y,z = pos
        l = QCanvasText(canvas)
        l.setText(text)
        l.setColor(Qt.black)
        l.setFont(QFont('Times',12,50))
        painter = QPainter()
        self.setX(float(x))
        self.setY(float(y))
        self.setZ(float(1))
        l.show()
        return l

class Line(QtCore.QLineF):
    def __init__(self,display, points, painter,color=(0,0,0)):
        self.parent.__init__(points[0],points[1])

        if self.length() == 0.0:
            return
        #draw Line
        painter.setPen(QtGui.QPen(QtCore.Qt.black, 1, QtCore.Qt.SolidLine, QtCore.Qt.RoundCap, QtCore.Qt.RoundJoin))
        painter.drawLine(self)

class Edge(EN.Edge):
    def __init__(self,sourcenode,destnode):
        self.parent.__init__(sourcenode,destnode)

class Map:
    """
    Draws maps as poligons
    """
    def __init__(self,fname, canvas):
        #self.display = visual.display(title='Brasil', ambient=0.5)
        self.canvas = canvas
        self.Reader(fname)


    def Reader(self, fname):
        """
        Reads Grass 5.x ascii vector files.
        """
        f = open(fname,'r')
        all = f.readlines()
        f.close()
        pol = []
        inicioP = [all.index(i) for i in all if i.startswith('B')]
        for i in inicioP:
            size = int(all[i].split()[-1])
            a = [i.split() for i in all[i+1:i+1+size]]
            #print a
            for n,i in enumerate(a[:-1]):
                x1,y1,x2,y2 = float(i[1]),float(i[0]),float(a[n+1][1]),float(a[n+1][0])
                Line(self.canvas,(850+10*x1,-10*y1+50,850+10*x2,-10*y2+50))
            #self.dbound(pta) #Draws the polygon
        self.canvas.update()

    def dbound(self,pol):
        """
        Draws a polygon
        pol: list of points forming the polygon
        canvas: Canvas where it'll be drawn
        """
        P = QCanvasPolygon(self.canvas)
        P.setPen(QPen(Qt.black,10))
        P.setPoints(pol)
        P.show()


if __name__ == "__main__":
    app = QtGui.QApplication(sys.argv)
    QtCore.QObject.connect(app,QtCore.SIGNAL("lastWindowClosed()"),app,QtCore.SLOT("quit()"))
    w = MainWindow()
#    app.setMainWidget(w)
    w.show()
    sys.exit(app.exec_())
    



# -*- coding:utf8 -*-
import ez_setup
ez_setup.use_setuptools()
from setuptools import setup
from glob import glob
#from distutils.core import setup

demos = glob('demos/*')
try:
    demos.remove('demos/CVS')
except: pass

setup(name='epigrass',
    version='2.0',
    author = 'Flavio Codeco Coelho, Claudia Torres Codeco',
    author_email = 'fccoelho@gmail.com',
    maintainer = 'Flavio Codeco Coelho',
    maintainer_email = 'fccoelho@gmail.com',
    url = 'http://epigrass.sourceforge.net',
    description = 'Epidemiological Geo-Referenced Analysis and Simulation System',
    long_description = 'EpiGrass is a simulator of epidemics over networks.  Its is a scientific tool created for simulations and scenario analysis in Network epidemiology.',
    download_url = 'http://sourceforge.net/project/showfiles.php?group_id=128000',
    license = 'GPL',
    packages = ['Epigrass'],
    entry_points = {
        'console_scripts': [
            'epirunner = Epigrass.manager:main',
        ],
        'gui_scripts': [
            'epigrass = Epigrass.epigrass:main',
            'epgeditor= Epigrass.epgeditor:main'
        ]
    }, 

    include_package_data = True,
    package_data = {'':['INSTALL','README','COPYING','epigrass.desktop']},
    #data_files = [('/usr/share/pixmaps',['egicon.png']),('/usr/share/doc/epigrass/demos',demos),('/usr/share/doc/epigrass/',['docs/build/latex/Epigrass.pdf']),('/usr/share/applications',['epigrass.desktop'])]
    )



#! /usr/bin/env python

# embedding_in_qt.py --- Simple Qt application embedding matplotlib canvases
#
# Copyright (C) 2005 Florent Rougon
#
# This file is an example program for matplotlib. It may be used and
# modified with no restriction; raw copies as well as modified versions
# may be distributed without limitation.

import sys, os, random
from qt import *

from matplotlib.numerix import arange, sin, pi
from matplotlib.backends.backend_qtagg import FigureCanvasQTAgg as FigureCanvas
from matplotlib.figure import Figure
from pylab import plot

# This seems to be what PyQt expects, according to the examples shipped in
# its distribution.
TRUE  = 1
FALSE = 0

progname = os.path.basename(sys.argv[0])
progversion = "0.1"

# Note: color-intensive applications may require a different color allocation
# strategy.
#QApplication.setColorSpec(QApplication.NormalColor)
#app = QApplication(sys.argv)

class MyMplCanvas(FigureCanvas):
    """Ultimately, this is a QWidget (as well as a FigureCanvasAgg, etc.)."""
    def __init__(self, parent=None, width=5, height=4, dpi=100):
        self.fig = Figure(figsize=(width, height), dpi=dpi)
        self.axes = self.fig.add_subplot(111)
        # We want the axes cleared every time plot() is called
        self.axes.hold(False)
        self.plot = plot

        #self.compute_initial_figure()

        FigureCanvas.__init__(self, self.fig)
        self.reparent(parent, QPoint(0, 0))

        FigureCanvas.setSizePolicy(self,
                                   QSizePolicy.Expanding,
                                   QSizePolicy.Expanding)
        FigureCanvas.updateGeometry(self)

    def sizeHint(self):
        w, h = self.get_width_height()
        return QSize(w, h)

    def minimumSizeHint(self):
        return QSize(10, 10)





"""
This module uses the GDAL and OGR Library to read maps in vaious formats and
export the results of Epigrass simulations to the formats supported by these libraries

copyright 2007 by Flavio Codeco Coelho
Licensed under the GPL.
"""
import locale, os#, pylab
from osgeo  import ogr,  gdal
from xml.dom import minidom, Node
from matplotlib.colors import  rgb2hex, LogNorm
from matplotlib import cm
class World:
    def __init__(self,filename,namefield='name',geocfield='geocode',outdir = '.'):
        '''
        Instantiate a world object.
        Filename points to a file supported by OGR
        '''
        self.geocfield = geocfield
        self.namefield = namefield
        self.outdir = outdir
        self.ds =ogr.Open(filename)
        self.name = self.ds.GetName()
        self.driver = self.ds.GetDriver()
        self.centroids = []#centroid list (x,y,z) tuples
        self.centdict = {} #keys are geocode, values are (x,y,z) tuples
        self.geomdict = {} #keys are geocode, values are geometries
        self.nlist = [] #nodelist: feature objects
        self.nodesource = False #True if Node datasource has been created
        self.edgesource = False #True if Edge datasource has been created
        self.datasource = False #True if Data datasource has been created
        self.layerlist = self.getLayerList()
    def getLayerList(self):
        """
        returns a list with the
        available layers by name
        """
        nlay = self.ds.GetLayerCount()
        ln = []
        for i in xrange(nlay):
            l = self.ds.GetLayer(i)
            ln.append(l.GetName())
        return ln

    def drawLayer(self,L):
        '''
        Draws a polygon layer using pylab
        '''
        N = 0
        L.ResetReading()
        feat = L.GetNextFeature()
        while feat is not None:
            field_count = L.GetLayerDefn().GetFieldCount()
            geo = feat.GetGeometryRef()
            if geo.GetGeometryCount()<2:
                g1 = geo.GetGeometryRef( 0 )
                x =[g1.GetX(i) for i in range(g1.GetPointCount()) ]
                y =[g1.GetY(i) for i in range(g1.GetPointCount()) ]
                pylab.plot(x,y,'-',hold=True)
            for c in range( geo.GetGeometryCount()):
                        ring = geo.GetGeometryRef ( c )
                        for cnt in range( ring.GetGeometryCount()):
                            g1 = ring.GetGeometryRef( cnt )
                            x =[g1.GetX(i) for i in range(g1.GetPointCount()) ]
                            y =[g1.GetY(i) for i in range(g1.GetPointCount()) ]
                            pylab.plot(x,y,'-',hold=True)
            feat = L.GetNextFeature()
        pylab.xlabel("Longitude")
        pylab.ylabel("Latitude")
        pylab.grid(True)
        pylab.show()

    def getNodeList(self,l):
        '''
        Returns the centroid coordinates
        l is an OGR layer.
        '''
        self.nlist=[]
        f = l.GetNextFeature()
        while f is not None:
            g = f.GetGeometryRef()
            self.geomdict[f.GetFieldAsInteger(self.geocfield)] = g
            try:
                c = g.Centroid()
                self.nlist.append(f)
                self.centdict[f.GetFieldAsInteger(self.geocfield)] = (c.GetX(),c.GetY(),c.GetZ())
            except: #in case feature is not a polygon
                print  f.GetFID(),g.GetGeometryType()
            f = l.GetNextFeature()
        #print (2600501 in self.centdict)


    def createNodeLayer(self):
        """
        Creates a new shape file to represent network nodes.
        The node layer will be based on the centroids of the
        polygons belonging to the map layer associated with this
        world instance.
        namefield - name of the field containing polygon name
        geocfield - name of the field containing geocode
        """
        # Creates a new shape file to hold the data
        if os.path.exists(os.path.join(self.outdir,'Nodes.shp')):
            os.remove(os.path.join(self.outdir,'Nodes.shp'))
            os.remove(os.path.join(self.outdir,'Nodes.shx'))
            os.remove(os.path.join(self.outdir,'Nodes.dbf'))
        if not self.nodesource:
            dsn = self.driver.CreateDataSource(os.path.join(self.outdir,'Nodes.shp'))
            self.nodesource = dsn
            nl = dsn.CreateLayer("nodes",geom_type=ogr.wkbPoint)
        #Create the fields
        fi1 = ogr.FieldDefn("name")
        fi2 = ogr.FieldDefn("geocode",field_type=ogr.OFTInteger)
        fi3 = ogr.FieldDefn("x",field_type=ogr.OFTString)
        fi4 = ogr.FieldDefn("y",field_type=ogr.OFTString)
        nl.CreateField(fi1)
        nl.CreateField(fi2)
        nl.CreateField(fi3)
        nl.CreateField(fi4)
        #Add the features (points)
        for f in self.nlist:
            gc = f.GetFieldAsInteger(self.geocfield)
            x = self.centdict[gc][0]
            y = self.centdict[gc][1]
            fe = ogr.Feature(nl.GetLayerDefn())
            name = f.GetField(self.namefield)
            fe.SetField('name',f.GetField(self.namefield))
            fe.SetField('geocode',gc)
            fe.SetField('x',str(x))
            fe.SetField('y',str(y))
            pt = ogr.Geometry(type=ogr.wkbPoint)
            pt.AddPoint(x,y)
            fe.SetGeometryDirectly(pt)
            nl.CreateFeature(fe)
        nl.SyncToDisk()

    def createEdgeLayer(self,elist):
        """
        Creates a new layer with edge information.
        elist is a list of tuples:
        (sgeoc,dgeoc,fsd,fds)
        """
        # Creates a new shape file to hold the data
        if os.path.exists(os.path.join(self.outdir,'Edges.shp')):
            os.remove(os.path.join(self.outdir,'Edges.shp'))
            os.remove(os.path.join(self.outdir,'Edges.shx'))
            os.remove(os.path.join(self.outdir,'Edges.dbf'))
        if not self.edgesource:
            dse = self.driver.CreateDataSource(os.path.join(self.outdir,'Edges.shp'))
            self.edgesource = dse
            el = dse.CreateLayer("edges",geom_type=ogr.wkbLineString)
        #Create the fields
        fi1 = ogr.FieldDefn("source_geocode",field_type=ogr.OFTInteger)
        fi2 = ogr.FieldDefn("dest_geocode",field_type=ogr.OFTInteger)
        fi3 = ogr.FieldDefn("flowSD",field_type=ogr.OFTReal)
        fi3.SetPrecision(12)
        fi4 = ogr.FieldDefn("flowDS",field_type=ogr.OFTReal)
        fi4.SetPrecision(12)
        el.CreateField(fi1)
        el.CreateField(fi2)
        el.CreateField(fi3)
        el.CreateField(fi4)
        #Add the features (lines)
        for e in elist:
            #print "setting edge fields"
            fe = ogr.Feature(el.GetLayerDefn())
            fe.SetField('source_geocode',e[0])
            fe.SetField('dest_geocode',e[1])
            fe.SetField('flowSD',e[2])
            fe.SetField('flowSD',e[3])
            line = ogr.Geometry(type=ogr.wkbLineString)
            try:
                #print "creating edge lines"
                line.AddPoint(self.centdict[int(e[0])][0],self.centdict[int(e[0])][1])
                line.AddPoint(self.centdict[int(e[1])][0],self.centdict[int(e[1])][1])
                fe.SetGeometryDirectly(line)
                el.CreateFeature(fe)
            except: #node not in centdict
                pass
        el.SyncToDisk()

    def createDataLayer(self,varlist, data):
        """
        Creates a new shape to contain data about nodes.
        varlist is the list of fields names associated with
        the nodes.
        data is a list of lists whose first element is the geocode
        and the remaining elements are values of the fields, in the
        same order as they appear in varlist.
        """
        if os.path.exists(os.path.join(self.outdir,'Data.shp')):
            os.remove(os.path.join(self.outdir,'Data.shp'))
            os.remove(os.path.join(self.outdir,'Data.shx'))
            os.remove(os.path.join(self.outdir,'Data.dbf'))
        # Creates a new shape file to hold the data
        if not self.datasource:
            dsd = self.driver.CreateDataSource(os.path.join(self.outdir,'Data.shp'))
            self.datasource = dsd
            dl = dsd.CreateLayer("sim_results",geom_type=ogr.wkbPolygon)
        #Create the fields
        fi1 = ogr.FieldDefn("geocode",field_type=ogr.OFTInteger)
        dl.CreateField(fi1)
        for v in varlist:
            #print "creating data fields"
            fi = ogr.FieldDefn(v,field_type=ogr.OFTString)
            fi.SetPrecision(12)
            dl.CreateField(fi)

        #Add the features (points)
        for n,l in enumerate(data):
            #Iterate over the lines of the data matrix.
            gc = l[0]
            try:
                geom = self.geomdict[gc]
                if geom.GetGeometryType() != 3: continue
                #print geom.GetGeometryCount()
                fe = ogr.Feature(dl.GetLayerDefn())
                fe.SetField('geocode',gc)
                for v,d in zip (varlist,l[1:]):
                    #print v,d
                    fe.SetField(v,str(d))
                #Add the geometry
                #print "cloning geometry"
                clone = geom.Clone()
                #print geom
                #print "setting geometry"
                fe.SetGeometry(clone)
                #print "creating geom"
                dl.CreateFeature(fe)
            except: #Geocode not in polygon dictionary
                pass
            dl.SyncToDisk()

    def genSitesFile(self,fname):
        """
        This method generate a sites
        csv file from the nodes extracted from the
        map.
        """
        f = open(fname,"w")
        for fe in self.nlist:
            gc = fe.GetFieldAsInteger(self.geocfield)
            x = self.centdict[gc][0]
            y = self.centdict[gc][1]
            name = fe.GetField(self.namefield)
            line = "%s,%s,%s,%s\n"%(x,y,name,gc)
            #fe.SetField('name',f.GetField(self.namefield))
            f.write(line)
        f.close()


    def closeSources(self):
        """
        Close the data sources so that data is flushed and and files are closed
        """
        if self.nodesource:
            self.nodesource.Destroy()
            #print "closed node files"
        if self.edgesource:
            self.edgesource.Destroy()
            #print "closed edge files"
        if self.datasource:
            self.datasource.Destroy()
            #print "closed data files"


class KmlGenerator:
    """
        Generate a KML file for displaying data on 
        Google Maps/Earth
    """
    def __init__(self):
        self.doc = None
        self.dnode = None #document node in the DOM
        self.genRoot()
    def genRoot(self):
        """
        Generate a KML file root.
        """
        self.kmldoc = doc = minidom.Document()
        kml = doc.createElement("kml")
        kml.setAttribute("xmlns","http://earth.google.com/kml/2.1")
        doc.appendChild(kml)
        d = doc.createElement("Document")
        kml.appendChild(d)
        nel = doc.createElement("name")
        name = doc.createTextNode("KML Epigrass data file")
        nel.appendChild(name)
        d.appendChild(nel)
        desc = doc.createElement("description")
        d.appendChild(desc)
        desc.appendChild(doc.createTextNode("Polygons with data"))
        self.dnode = d

    def addNodes(self,layer,names=None):
        """
        Adds a node to the document.
        d is the document element KML dom object.
        layer is a layer with polygons
        names is a dictionary of names indexed by geocode(int)
        """
        doc = self.dnode
        jet  = cm.get_cmap("jet",50)
        layer.ResetReading()
        while 1:
            f = layer.GetNextFeature()
            if not f:#exit after the last feature
                break
            prevalence = float(f.GetField("prevalence"))
            rgba = jet(prevalence)
            bgrcol = list(rgba[:-1]) #rgb(list)
            bgrcol.reverse() #turn it into bgr
            hexcol = "#80"+rgb2hex(bgrcol)[1:] #abgr Alpha set to 128
            g = f.GetGeometryRef()
            if g.GetGeometryType() == 3:
                if not names:
                    name = ""
                else:
                    try:
                        name = names[f.GetFieldAsInteger("geocode")]
                    except KeyError:
                        print f.GetFieldAsInteger("geocode")
                        name = ""
                description = "Prevalence: %s;\nTotal cases: %s;\nImported Cases: %s;"%(prevalence,f.GetField("totalcases"),f.GetField("arrivals"))
                locale.setlocale(locale.LC_ALL,"C") #avoids conversion of decimal points
                gml = g.ExportToGML() #extract the coordinates from the GML representation
                coords = gml.split('<gml:coordinates>')[1].split('</gml:coordinates>')[0]
                coords  = " ".join([i+",0" for i in coords.split(" ")]) #add z coordinate
                #create the kml elements
                #placemark and sub elements
                pm=self.kmldoc.createElement("Placemark")
                nm = self.kmldoc.createElement("name")
                nm.appendChild(self.kmldoc.createTextNode(name))
                desc = self.kmldoc.createElement("description")
                desc.appendChild(self.kmldoc.createTextNode(description))
                pm.appendChild(nm)
                pm.appendChild(desc)
                #style and subelements
                st=self.kmldoc.createElement("Style")
                pm.appendChild(st)
                ps = self.kmldoc.createElement("PolyStyle")
                color = self.kmldoc.createElement("color")
                color.appendChild(self.kmldoc.createTextNode(hexcol))
                ps.appendChild(color)
                fill = self.kmldoc.createElement("fill")
                fill.appendChild(self.kmldoc.createTextNode("1"))
                ps.appendChild(fill)
                outline = self.kmldoc.createElement("outline")
                outline.appendChild(self.kmldoc.createTextNode("1"))
                ps.appendChild(outline)
                st.appendChild(ps)
                doc.appendChild(pm)
                #Multigeometry
                mg = self.kmldoc.createElement("MultiGeometry")
                pm.appendChild(mg)
                polygon = self.kmldoc.createElement("Polygon")
                mg.appendChild(polygon)
                ob = self.kmldoc.createElement("outerBoundaryIs")
                polygon.appendChild(ob)
                linr = self.kmldoc.createElement("LinearRing")
                ob.appendChild(linr)
                coordin = self.kmldoc.createElement("coordinates")
                linr.appendChild(coordin)
                coordin.appendChild(self.kmldoc.createTextNode(coords))
    def writeToFile(self,dir):
        """
        Writes the kml file to disk
        """
        fullpath = os.path.join(dir,"Data.kml")
        f=open(fullpath,"w")
        f.write(self.kmldoc.toxml())
        f.close()

if __name__=="__main__":
    # opening data source
    w = World('riozonas_LatLong.shp','nome_zonas','zona_trafe')
    layer = w.ds.GetLayerByName(w.layerlist[0])
    w.getNodeList(layer)
    w.drawLayer(layer)
    w.createNodeLayer()
    w.nodesource.Destroy() #flush data to disk
##    k = KmlGenerator()
##    k.addNodes(w.datasource.GetLayer(0))
##    k.writeToFile()
